@@ -1,4 +1,8 @@
+"""Embedding layer.
+"""
 from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
 
 from .. import backend as K
 from .. import initializers
@@ -75,7 +79,6 @@ class Embedding(Layer):
                  mask_zero=False,
                  input_length=None,
                  **kwargs):
-        kwargs['dtype'] = 'int32'
         if 'input_shape' not in kwargs:
             if input_length:
                 kwargs['input_shape'] = (input_length,)
@@ -98,7 +101,8 @@ class Embedding(Layer):
             initializer=self.embeddings_initializer,
             name='embeddings',
             regularizer=self.embeddings_regularizer,
-            constraint=self.embeddings_constraint)
+            constraint=self.embeddings_constraint,
+            dtype=self.dtype)
         self.built = True
 
     def compute_mask(self, inputs, mask=None):
@@ -108,11 +112,25 @@ class Embedding(Layer):
             return K.not_equal(inputs, 0)
 
     def compute_output_shape(self, input_shape):
-        if not self.input_length:
-            input_length = input_shape[1]
+        if self.input_length is None:
+            return input_shape + (self.output_dim,)
         else:
-            input_length = self.input_length
-        return (input_shape[0], input_length, self.output_dim)
+            # input_length can be tuple if input is 3D or higher
+            if isinstance(self.input_length, (list, tuple)):
+                in_lens = list(self.input_length)
+            else:
+                in_lens = [self.input_length]
+            if len(in_lens) != len(input_shape) - 1:
+                ValueError('"input_length" is %s, but received input has shape %s' %
+                           (str(self.input_length), str(input_shape)))
+            else:
+                for i, (s1, s2) in enumerate(zip(in_lens, input_shape[1:])):
+                    if s1 is not None and s2 is not None and s1 != s2:
+                        ValueError('"input_length" is %s, but received input has shape %s' %
+                                   (str(self.input_length), str(input_shape)))
+                    elif s1 is None:
+                        in_lens[i] = s2
+            return (input_shape[0],) + tuple(in_lens) + (self.output_dim,)
 
     def call(self, inputs):
         if K.dtype(inputs) != 'int32':
